/* C version of fixavi.pas */

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <assert.h>

#define BUFSIZE 10240
typedef unsigned char byte;
/* "word" must be 2 bytes long; change this if "short" is not 2-byte on your system */
typedef unsigned short word;

void x_abort(FILE *f,const char *fmt,...) {
if (f!=NULL) fclose(f);
va_list ap;
va_start(ap,fmt);
vfprintf(stderr,fmt,ap);
va_end(ap);
exit(1);
}

main(int argc,char *argv[]) {
byte buffer[BUFSIZE];
size_t nbytes;
FILE *f;
char *curptr;
int found=0; int ok=0; int stop=0;
byte *endptr;

/* these will fail if the "byte" and "word" types are not required sizes */
assert(sizeof(byte)==1);
assert(sizeof(word)==2);

if (argc<2) x_abort(NULL,"Usage: %s filename.avi\n",argv[0],NULL);
f=fopen(argv[1],"r+");
if (f==NULL) x_abort(NULL,"Can't open file %s\n",argv[1],NULL);
nbytes=fread(buffer,sizeof(byte),sizeof(buffer),f);
if (nbytes<=0) x_abort(f,"Error reading file\n",NULL);
curptr=(char *)buffer; endptr=buffer+nbytes-4;
while (!stop && (byte *)curptr<=endptr) {
  if ((byte *)curptr==buffer && strncmp(curptr,"RIFF",4)!=0) x_abort(f,"Improper file format\n",NULL);
  if (strncmp(curptr,"auds",4)==0) found=1;
  if (found && *((word *)curptr)==11024) {
    ok=1;
    *((word *)curptr)=11025;
  }
  if (found && strncmp(curptr,"LIST",4)==0) stop=1;
  curptr += 4;
}
if (ok) {
  if (fseek(f,0,SEEK_SET)!=0) x_abort(f,"Error writing file\n",NULL);
  if (fwrite(buffer,sizeof(byte),nbytes,f)!=nbytes) x_abort(f,"Error writing file\n",NULL);
  printf("File fixed OK\n");
} else {
  if (found)
    printf("Nothing found to fix (file already OK?)\n");
  else
    printf("Can't find audio track - improper file format\n");
}
fclose(f);
exit(0);
/* end of program */
} 

